using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Text;


namespace Platform
{
    abstract class Actor 
    {
        public Vector2 position;
        public Vector2 velocity;
        public Vector2 acc;
        public states state;
       
        public Vector2 spherepos;
        public float sphererad;

        protected Color tintColor;
        
        private float anim = 0;
        
        protected Game1 game;
        public float friction;

        protected Texture2D[] textureArr2;

        protected Texture2D[] textureArr;
        public Texture2D[] texture
        {
            get
            {
                return textureArr;
            }
            set
            {
                textureArr = value;
                if(textureArr!=null)
                {
                    sphererad = (float)Math.Max(textureArr[0].Height, textureArr[0].Width);
                    sphererad = (float)Math.Pow(sphererad, 2);
                }
            }
        }

        
        public SpriteBatch sprite;

        public Actor(Game1 g,Texture2D[] tex,SpriteBatch spr)
        {
            game = g;
            textureArr = tex;
            textureArr2 = game.smokeText;
            sprite = spr;
            position = Vector2.Zero;
            velocity = Vector2.Zero;
            spherepos = Vector2.Zero;
            acc = game.gravity;
            state = states.IN_AIR;
            friction = 0.95f;
            tintColor = Color.White;
        }

        public virtual void update()
        {
            calculatePos();
            switch (state) {

                case states.ON_GROUND:
                        velocity.X += acc.X;
                        velocity.Y = 0.0f;
                        position += velocity;
                        velocity.X = friction * velocity.X;
                        break;
                case states.IN_AIR:
                        velocity += acc;
                        position += velocity;
                        acc = game.gravity;
                        break;     
                case states.SMOKE:
                        position += velocity;
                        break;
                    
            }
            checkCollision();
        }

        public void draw(GameTime gameTime)
        {

            switch (state)
            {
                case states.ON_GROUND:
                    sprite.Begin(SpriteBlendMode.AlphaBlend);

                    anim += (float)(10*Math.Abs(velocity.X)*gameTime.ElapsedGameTime.TotalSeconds);

                    anim %= texture.Length;

                    /*
                    tic += (int)((velocity.X * 0.6f));
                    if (tic % 10 == 0)
                    {
                        anim ++;
                        if (anim == texture.Length) anim = 0;
                        tic = 0;
                    }
                     */
                    sprite.Draw(textureArr[(int)anim], position, tintColor);
                    sprite.End();
                    break;
                case states.IN_AIR:
                    sprite.Begin(SpriteBlendMode.AlphaBlend);
                    sprite.Draw(textureArr[(int)anim], position, tintColor);
                    sprite.End();
                    break;
                case states.SMOKE:
                    sprite.Begin(SpriteBlendMode.AlphaBlend);
                    sprite.Draw(textureArr2[0], position, tintColor);
                    sprite.End();
                    break;
                default:
                    break;
            }
        }

        protected void checkCollision()
        {



         //   if (position.Y > 400f)
           // {
           //     outofBoundsFunc(1);
            //}
        

            if (position.X+texture[0].Width > game.ScreenW)
            {
                outofBoundsFunc(2);

            }
            else if(position.X < 0)
            {
                outofBoundsFunc(3);

            }
            else if (position.Y > game.ScreenH)
            {
                outofBoundsFunc(4);
            }
            if (game.back.intersctionTest(position + new Vector2(0,textureArr[0].Height)))
            {
                outofBoundsFunc(1); 
            }
            else outofBoundsFunc(5);
            spherepos.X = position.X + textureArr[(int)anim].Width * 0.5f;
            spherepos.Y = position.Y + textureArr[(int)anim].Height * 0.5f;    
        }

        public abstract void checkCollision(Actor a);

        protected abstract void calculatePos();

        protected abstract void outofBoundsFunc(int i);
    }
}
