
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Platform
{
    enum states
    {
        IN_AIR,ON_GROUND,DELETE,HIT,HIT_BY_ENEMY,HIT_BY_BULLET,SMOKE
    }

    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Actorlist actors;
        
        Player player;

        float playerspeed = 0.3f;
        public Color background;
        public Vector2 gravity = new Vector2(0f,0.098f);
        public int ScreenH;
        public int ScreenW;

        Texture2D[] bulletText;
        SpriteBatch bulletSprite;
        Texture2D[] evilText;
        SpriteBatch evilSprite;
        public Texture2D[] smokeText;
        public SpriteBatch smokeSprite;

        public float time;

        public Random gameRandom;

        public CollideBackground back;

       

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
            actors = new Actorlist(this);
            player = new Player(this, null, null);
            actors.Add(player);
            gameRandom = new Random();
            background = Color.FloralWhite;
        }

        #region Installization and stuff
        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            Random r = new Random();
            // TODO: Add your initialization logic here
            for (int i = 0; i <5; i++)
            {
                actors.Add(new Enemy(this,null,null,player));
                actors[i].position = new Vector2(1000 * (float)r.NextDouble(), 0);
            }
            this.IsFixedTimeStep = true;
         //   this.TargetElapsedTime = new TimeSpan(200000);
            base.Initialize();
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                
                Texture2D[] t = new Texture2D[2];
                t[0] = this.content.Load<Texture2D>("texture/gubbe");
                t[1] = this.content.Load<Texture2D>("texture/gubbe2");
                player.sprite = new SpriteBatch(graphics.GraphicsDevice);
                player.texture = t;
                player.rightTextArr = t;
                player.leftTextArr = new Texture2D[2];
                player.leftTextArr[0] = this.content.Load<Texture2D>("texture/gubber");
                player.leftTextArr[1] = this.content.Load<Texture2D>("texture/gubbe2r");
                

                Texture2D b = this.content.Load<Texture2D>("texture/t");
                back = new CollideBackground(this,b,new SpriteBatch(graphics.GraphicsDevice));
                
                evilText = new Texture2D[2];
                
                evilText[0] = this.content.Load<Texture2D>("texture/evil");
                evilText[1] = this.content.Load<Texture2D>("texture/evil2");
                evilSprite = new SpriteBatch(graphics.GraphicsDevice);

                foreach (Actor e in actors)
                {
                    if (e.GetType().Equals(typeof(Enemy)))
                    {
                        e.texture = evilText;
                        e.sprite = evilSprite;
                    }
                }

                // TODO: Load any ResourceManagementMode.Automatic content
                ScreenH = graphics.PreferredBackBufferHeight;
                ScreenW = graphics.PreferredBackBufferWidth;

                bulletText = new Texture2D[1];
                bulletText[0] = this.content.Load<Texture2D>("texture/bullet");
                bulletSprite = new SpriteBatch(graphics.GraphicsDevice);

                smokeSprite = new SpriteBatch(graphics.GraphicsDevice);
                smokeText = new Texture2D[1];
                smokeText[0] = this.content.Load<Texture2D>("texture/smoke");
                 
                
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
            }
        }
        #endregion

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            time = (float)gameTime.ElapsedGameTime.Milliseconds;
            // Allows the default game to exit on Xbox 360 and Windows

            UpdateInput();
            actors.collision(0);

            foreach (Actor a in actors)
                        a.update();
                   
            // TODO: Add your update logic here
            
            base.Update(gameTime);
            
            GC.Collect();
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(background);
            back.draw();
            foreach (Actor a in actors) a.draw(gameTime);
            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }

        protected void UpdateInput()
        {
            //get the gamepad state
            GamePadState currentState = GamePad.GetState(PlayerIndex.One);
            if (currentState.IsConnected)
            {
                if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                    this.Exit();

                if (player.state == states.ON_GROUND)
                {
                    player.acc.X = playerspeed * currentState.ThumbSticks.Left.X;
                    if (currentState.ThumbSticks.Left.X < 0)
                        player.turnLeft();
                    else if (currentState.ThumbSticks.Left.X > 0)
                        player.turnRight();

                    if (currentState.Buttons.A == ButtonState.Pressed)
                    {
                        player.acc.Y -= 5;
                        player.state = states.IN_AIR;
                    }
                }
                else
                {
                    if (currentState.ThumbSticks.Left.X < 0)
                        player.turnLeft();
                    else if (currentState.ThumbSticks.Left.X > 0)
                        player.turnRight();

                    player.acc.X = playerspeed * 0.1f * currentState.ThumbSticks.Left.X;
                }

                if (currentState.Buttons.X == ButtonState.Pressed)
                {
                    Vector2 speed = Vector2.Zero;

                    if (player.goingRight)
                        speed.X = 1;
                    else
                        speed.X = -1;

                        speed.Y = -0.25f;
                        speed = speed * 10f;

                        actors.Add(new Bullet(this, bulletText, bulletSprite, speed, player));
                        GamePad.SetVibration(PlayerIndex.One, 0,0.5f);
                 }
                else if(currentState.Buttons.X == ButtonState.Released)
                 GamePad.SetVibration(PlayerIndex.One, 0, 0);
                //in case you get lost, press A to warp back to the center
            }
            else 
            {
                if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                    this.Exit();

                KeyboardState key = Keyboard.GetState();
                if (player.state == states.ON_GROUND)
                {
                    if (key.IsKeyUp(Keys.Left) && key.IsKeyUp(Keys.Right))
                    {
                        player.acc.X = 0;
                    }

                    if (key.IsKeyDown(Keys.Left))
                    {
                        player.acc.X = -playerspeed;
                        player.turnLeft();
                    }
                    else if (key.IsKeyDown(Keys.Right))
                    {
                        player.acc.X = playerspeed;
                        player.turnRight();
                    }

                    if (key.IsKeyDown(Keys.Up))
                    {
                        player.acc.Y -= 5;
                        player.state = states.IN_AIR;
                    }
                }
                else
                {
                    if (key.IsKeyUp(Keys.Left) && key.IsKeyUp(Keys.Right))
                    {
                        player.acc.X = 0;
                    }

                    if (key.IsKeyDown(Keys.Left))
                    {
                        player.acc.X = -playerspeed*0.1f;
                        player.turnLeft();
                    }
                    else if (key.IsKeyDown(Keys.Right))
                    {
                        player.acc.X = playerspeed*0.1f;
                        player.turnRight();
                    }
                }

                if(key.IsKeyDown(Keys.Space))
                {
                    Vector2 speed = Vector2.Zero;
                    if (player.goingRight)
                        speed.X = 1;
                    else
                        speed.X = -1;

                    speed.Y = -0.25f;
                    speed = speed * 10f;
                    actors.Add(new Bullet(this, bulletText, bulletSprite, speed, player));
                }
            }


            }
        }

    }
